@@ -11,6 +11,7 @@ from piperider_cli.assertion_generator import AssertionGenerator
 from piperider_cli.cloud_connector import CloudConnector
 from piperider_cli.compare_report import CompareReport
 from piperider_cli.configuration import PIPERIDER_WORKSPACE_PATH
+from piperider_cli.error import RecipeConfigException
 from piperider_cli.event import UserProfileConfigurator
 from piperider_cli.event.track import TrackCommand
 from piperider_cli.exitcode import EC_ERR_TEST_FAILED
@@ -156,6 +157,7 @@ def diagnose(**kwargs):
 @click.option('--dbt-run-results', is_flag=True, help='Associate with dbt run results.')
 @click.option('--report-dir', default=None, type=click.STRING, help='Use a different report directory.')
 @click.option('--upload', is_flag=True, help='Upload the report to the PipeRider Cloud.')
+@click.option('--project', default=None, type=click.STRING, help='Specify the project name to upload.')
 @click.option('--open', is_flag=True, help='Opens the generated report in the system\'s default browser')
 @add_options(debug_option)
 def run(**kwargs):
@@ -170,6 +172,7 @@ def run(**kwargs):
     dbt_list = kwargs.get('dbt_list')
     dbt_run_results = kwargs.get('dbt_run_results')
     force_upload = kwargs.get('upload')
+    project_name = kwargs.get('project')
 
     console = Console()
 
@@ -207,7 +210,7 @@ def run(**kwargs):
         if CloudConnector.is_login() and is_cloud_view:
             CloudConnector.upload_latest_report(report_dir=kwargs.get('report_dir'), debug=kwargs.get('debug'),
                                                 open_report=open_report, force_upload=force_upload,
-                                                auto_upload=auto_upload)
+                                                auto_upload=auto_upload, project_name=project_name)
 
         if not skip_report:
             GenerateReport.exec(None, kwargs.get('report_dir'), output, open_report, is_cloud_view)
@@ -255,6 +258,10 @@ def generate_report(**kwargs):
 @click.option('--tables-from', default='all',
               type=click.Choice(['all', 'target-only', 'base-only'], case_sensitive=False),
               help='Show table comparison from base or target.')
+@click.option('--upload', default=False, is_flag=True, help='Upload the report to PipeRider Cloud.')
+@click.option('--project', default=None, type=click.STRING,
+              help='Specify the project name to upload.')
+@click.option('--share', default=False, is_flag=True, help='Enable public share of the report to PipeRider Cloud.')
 @add_options(debug_option)
 def compare_reports(**kwargs):
     'Compare two existing reports selected in interactive mode or by option.'
@@ -264,9 +271,23 @@ def compare_reports(**kwargs):
     last = kwargs.get('last')
     datasource = kwargs.get('datasource')
     tables_from = kwargs.get('tables_from')
+    force_upload = kwargs.get('upload')
+    enable_share = kwargs.get('share')
+    project_name = kwargs.get('project')
+
+    if enable_share:
+        force_upload = True
+
+    if force_upload is True and CloudConnector.is_login() is False:
+        raise RecipeConfigException(
+            message='Please login to PipeRider Cloud first.',
+            hint='Run "piprider cloud login" to login to PipeRider Cloud.'
+        )
+
     CompareReport.exec(a=a, b=b, last=last, datasource=datasource,
                        report_dir=kwargs.get('report_dir'), output=kwargs.get('output'), tables_from=tables_from,
-                       debug=kwargs.get('debug', False))
+                       force_upload=force_upload, enable_share=enable_share, project_name=project_name,
+                       debug=kwargs.get('debug', False), show_progress=True)
 
 
 @cli.group('config', short_help='Manage the PipeRider configurations.')
@@ -333,7 +354,7 @@ def cloud(**kwargs):
 @click.option('--datasource', default=None, type=click.STRING, metavar='DATASOURCE_NAME',
               help='Specify the datasource.')
 @click.option('--project', default=None, type=click.STRING, metavar='PROJECT_NAME',
-              help='Specify the project to upload.')
+              help='Specify the project name to upload.')
 @add_options(debug_option)
 def upload_report(**kwargs):
     """
@@ -359,7 +380,7 @@ def upload_report(**kwargs):
               help='Show table comparison from base or target.')
 @click.option('--summary-file', default=None, type=click.STRING, help='Download the comparison summary markdown file.')
 @click.option('--project', default=None, type=click.STRING, metavar='PROJECT_NAME',
-              help='Specify the project to upload.')
+              help='Specify the project name to upload.')
 @add_options(debug_option)
 def cloud_compare_reports(**kwargs):
     """
@@ -381,6 +402,8 @@ def cloud_compare_reports(**kwargs):
 
 @cli.command(name='compare', short_help='Generate comparison report with the recipe.', cls=TrackCommand)
 @click.option('--recipe', default=None, type=click.STRING, help='Select a different recipe.')
+@click.option('--upload', default=False, is_flag=True, help='Upload the report to PipeRider Cloud.')
+@click.option('--share', default=False, is_flag=True, help='Enable public share of the report to PipeRider Cloud.')
 @click.option('--output', '-o', default=None, type=click.STRING, help='Directory to save the results.')
 @click.option('--summary-file', default=None, type=click.STRING, help='Output the comparison summary markdown file.')
 @add_options(debug_option)
@@ -391,34 +414,51 @@ def compare_with_recipe(**kwargs):
 
     recipe = kwargs.get('recipe')
     summary_file = kwargs.get('summary_file')
+    force_upload = kwargs.get('upload')
+    enable_share = kwargs.get('share')
+    debug = kwargs.get('debug', False)
+
+    if enable_share:
+        force_upload = True
+
+    if force_upload is True and CloudConnector.is_login() is False:
+        raise RecipeConfigException(
+            message='Please login to PipeRider Cloud first.',
+            hint='Run "piprider cloud login" to login to PipeRider Cloud.'
+        )
 
     ret = 0
     try:
-        RecipeExecutor.exec(recipe_name=recipe)
+        RecipeExecutor.exec(recipe_name=recipe, debug=debug)
         CompareReport.exec(a=None, b=None, last=True, datasource=None,
                            output=kwargs.get('output'), tables_from="all",
                            summary_file=summary_file,
-                           debug=kwargs.get('debug', False))
+                           force_upload=force_upload,
+                           enable_share=enable_share,
+                           debug=debug)
     except Exception as e:
         raise e
 
     return ret
 
 
+@cloud.command(short_help='Signup to PipeRider Cloud.', cls=TrackCommand)
+@add_options(debug_option)
+def signup(**kwargs):
+    ret = CloudConnector.signup(debug=kwargs.get('debug', False))
+    return ret
+
+
 @cloud.command(short_help='Login to PipeRider Cloud.', cls=TrackCommand)
 @click.option('--token', default=None, type=click.STRING, help='Specify the API token.')
-@click.option('--enable-auto-upload', default=None, is_flag=True, help='Enable auto upload.')
-@click.option('--disable-auto-upload', default=None, is_flag=True, help='Disable auto upload.')
+@click.option('--project', default=None, type=click.STRING, metavar='PROJECT_NAME',
+              help='Specify the default project name.')
 @add_options(debug_option)
 def login(**kwargs):
-    options = {
-        # True for enable, False for disable, None for not defined
-        'auto_upload': None,
-    }
-    if kwargs.get('enable_auto_upload'):
-        options['auto_upload'] = True
-    elif kwargs.get('disable_auto_upload'):
-        options['auto_upload'] = False
+    options = {}
+
+    if kwargs.get('project') is not None:
+        options['default_project'] = kwargs.get('project')
 
     ret = CloudConnector.login(api_token=kwargs.get('token'), options=options, debug=kwargs.get('debug', False))
     return ret
