@@ -18,6 +18,13 @@ SERVICE_ENV_SERVICE_KEY = 'PIPERIDER_API_SERVICE'
 yml = yaml.YAML()
 
 
+class PipeRiderProject(object):
+    def __init__(self, project: dict):
+        self.id = project.get('id')
+        self.name = project.get('name')
+        self.workspace_name = project.get('workspace_name')
+
+
 class CloudServiceHelper:
 
     def __init__(self):
@@ -85,7 +92,7 @@ class CloudServiceHelper:
         if not (self.api_token and self.api_service):
             return False, None
 
-        json_resp = self.http_get('/api/users/me')
+        json_resp = self.http_get('/api/v2/users/me')
         if json_resp and 'email' in json_resp:
             return True, json_resp
         return False, None
@@ -122,14 +129,33 @@ class PipeRiderCloud:
         self.service.api_token = None
         self.service.update_api_token()
 
+        # remove the default project when logout
+        self.update_config({'default_project': None})
+
+    def magic_signup(self, email: str, username: str = None):
+        if self.available:
+            return True
+        signup_url = self.service.url('/api/v2/credentials/signup')
+        payload = {'email': email, 'source': 'cli'}
+        if username:
+            payload['username'] = username
+        response = requests.post(
+            signup_url,
+            headers={'Content-type': 'application/json', 'Accept': 'text/plain'},
+            data=json.dumps(payload)
+        )
+        if response.status_code == 200:
+            return response.json()
+        return None
+
     def magic_login(self, email):
         if self.available:
             return True
-        login_url = self.service.url('/api/session/new')
+        login_url = self.service.url('/api/v2/session/new')
         response = requests.post(
             login_url,
             headers={'Content-type': 'application/json', 'Accept': 'text/plain'},
-            data=json.dumps({'email': email})
+            data=json.dumps({'email': email, 'source': 'cli'})
         )
         if response.status_code == 200:
             return response.json()
@@ -138,7 +164,7 @@ class PipeRiderCloud:
     def set_default_project(self, project_name):
         self.update_config({'default_project': project_name})
 
-    def get_default_project(self):
+    def get_default_project(self) -> PipeRiderProject:
         if not self.available:
             self.raise_error()
 
@@ -146,9 +172,9 @@ class PipeRiderCloud:
             name = self.config.get('default_project')
             project = self.get_project_by_name(name)
             if project:
-                return project.get('id')
+                return project
 
-        url = self.service.url('/api/projects')
+        url = self.service.url('/api/v2/workspaces')
         headers = self.service.auth_headers()
         response = requests.get(url, headers=headers)
 
@@ -159,20 +185,49 @@ class PipeRiderCloud:
             if project.get('id'):
                 # Legacy projects api response
                 if project.get('is_default'):
-                    return project.get('id')
+                    return PipeRiderProject(project)
             else:
                 # New projects api response
-                for p in project.get('projects', []):
-                    if p.get('is_default'):
-                        return p.get('id')
+                for p in project.get('projects', [])[:1]:
+                    return PipeRiderProject(p)
 
-    def list_reports(self, project_id, datasource=None):
+    def get_default_workspace_and_project(self):
         if not self.available:
             self.raise_error()
 
-        url = self.service.url(f'/api/projects/{project_id}/reports')
+        if self.config.get('default_project'):
+            name = self.config.get('default_project')
+            workspace_name, project_name = name.split('/')
+            return workspace_name, project_name
+
+        # Query workspace
+        url = self.service.url('/api/v2/workspaces')
+        headers = self.service.auth_headers()
+        response = requests.get(url, headers=headers)
+        if response.status_code != 200:
+            return None, None
+
+        response = response.json()
+        if len(response.get('data', [])) == 0:
+            return None, None
+        workspace = response.get('data')[0]
+        workspace_name = workspace.get('name')
+        project_name = None
+        for p in workspace.get('projects', [])[:1]:
+            project_name = p.get('name')
+            break
+
+        return workspace_name, project_name
+
+    def list_reports(self, project: PipeRiderProject, datasource=None):
+        if not self.available:
+            self.raise_error()
+
+        url = self.service.url(f'/api/v2/workspaces/{project.workspace_name}/projects/{project.name}/runs')
+
         if datasource:
-            url = self.service.url(f'/api/projects/{project_id}/reports?datasource={datasource}')
+            url = self.service.url(
+                f'/api/v2/workspaces/{project.workspace_name}/projects/{project.name}/runs?datasource={datasource}')
 
         headers = self.service.auth_headers()
         response = requests.get(url, headers=headers)
@@ -182,7 +237,7 @@ class PipeRiderCloud:
 
         return response.json()
 
-    def upload_report(self, file_path, show_progress=True, project_id=None):
+    def upload_run(self, file_path, show_progress=True, project: PipeRiderProject = None):
         # TODO validate project name
         if not self.available:
             self.raise_error()
@@ -200,8 +255,8 @@ class PipeRiderCloud:
             )
             m = MultipartEncoderMonitor(encoder, _upload_callback)
 
-            url = self.service.url(f'/api/projects/{project_id}/reports/upload') if project_id else self.service.url(
-                '/api/reports/upload')
+            url = self.service.url(
+                f'/api/v2/workspaces/{project.workspace_name}/projects/{project.name}/runs/upload')
             headers = self.service.auth_headers()
             headers['Content-Type'] = m.content_type
 
@@ -222,11 +277,12 @@ class PipeRiderCloud:
 
             return response.json()
 
-    def compare_reports(self, project_id, base_id: int, target_id: int, tables_from):
+    def compare_reports(self, base_id: int, target_id: int, tables_from, project: PipeRiderProject):
         if not self.available:
             self.raise_error()
 
-        url = self.service.url(f'/api/projects/{project_id}/reports/{base_id}/compare/{target_id}')
+        url = self.service.url(
+            f'/api/v2/workspaces/{project.workspace_name}/projects/{project.name}/runs/{base_id}/compare/{target_id}')
         headers = self.service.auth_headers()
         response = requests.post(url, data=json.dumps({'tables_from': tables_from}), headers=headers)
 
@@ -235,6 +291,20 @@ class PipeRiderCloud:
 
         return response.json()
 
+    def share_compare_report(self, workspace_name: str, project_name: str, base_id: int, target_id: int):
+        if not self.available:
+            self.raise_error()
+
+        url = self.service.url(
+            f'/api/v2/workspaces/{workspace_name}/projects/{project_name}/runs/{base_id}/compare/{target_id}/share')
+        headers = self.service.auth_headers()
+        response = requests.post(url, headers=headers)
+
+        if response.status_code != 200:
+            return None
+
+        return response.json()
+
     def raise_error(self):
         raise ValueError("Service not available or configuration invalid")
 
@@ -245,7 +315,7 @@ class PipeRiderCloud:
         if not self.available:
             self.raise_error()
 
-        url = self.service.url('/api/projects')
+        url = self.service.url('/api/v2/workspaces')
         headers = self.service.auth_headers()
         response = requests.get(url, headers=headers)
 
@@ -255,52 +325,39 @@ class PipeRiderCloud:
         data = response.json()
         output = []
 
-        def _parse_legacy_project_list(x):
-            output.append({
-                'id': x.get('id'),
-                'name': x.get('name'),
-                'is_default': x.get('is_default'),
-                'parent_type': 'personal',
-            })
+        if not data.get('success'):
+            return None
 
-        def _parse_projects_with_organization_list(x):
+        data = data.get('data')
+
+        def _parse_projects_with_workspace_list(x):
             for project in x.get('projects', []):
-                parent_type = 'organization' if project.get('organization_id') else 'personal'
-
                 p = {
                     'id': project.get('id'),
                     'name': project.get('name'),
-                    'is_default': project.get('is_default'),
-                    'parent_type': parent_type
+                    'workspace_name': x.get('name'),
+                    'workspace_display_name': x.get('display_name'),
                 }
-                if parent_type == 'organization':
-                    p['organization_name'] = x.get('name')
-                    p['organization_display_name'] = x.get('display_name')
                 output.append(p)
 
         for x in data:
-            if x.get('id'):
-                _parse_legacy_project_list(x)
-            else:
-                _parse_projects_with_organization_list(x)
+            _parse_projects_with_workspace_list(x)
         return output
 
-    def get_project_by_name(self, name):
-        project_name = None
-        organization_name = None
+    def get_project_by_name(self, name) -> PipeRiderProject:
+        workspace_name = None
         if '/' in name:
-            organization_name, project_name = name.split('/')
+            workspace_name, project_name = name.split('/')
         else:
             project_name = name
 
         projects = self.list_projects()
         for project in projects:
-            if project.get('name') == project_name and project.get('organization_name') == organization_name:
-                return project
-
+            if project.get('name') == project_name and project.get('workspace_name') == workspace_name:
+                return PipeRiderProject(project)
         return None
 
 
 if __name__ == '__main__':
     cloud = PipeRiderCloud()
-    cloud.upload_report(json.dumps(dict(foo='barbar')))
+    cloud.upload_run(json.dumps(dict(foo='barbar')))
