@@ -1,3 +1,4 @@
+import json
 import os
 import sys
 import webbrowser
@@ -11,7 +12,7 @@ from rich.prompt import Prompt
 from rich.table import Table
 
 from piperider_cli import datetime_to_str, open_report_in_browser, str_to_datetime
-from piperider_cli.cloud import PipeRiderCloud
+from piperider_cli.cloud import PipeRiderCloud, PipeRiderProject
 from piperider_cli.compare_report import CompareReport, RunOutput
 from piperider_cli.datasource import FANCY_USER_INPUT
 from piperider_cli.filesystem import FileSystem
@@ -19,9 +20,10 @@ from piperider_cli.filesystem import FileSystem
 console = Console()
 piperider_cloud = PipeRiderCloud()
 
+WORKSPACE_PROJECT_NAME_REGEX = r'^[a-zA-Z0-9-_]+$'
 
-def ask_login_info() -> str:
-    console.print('Please provide available email account to login')
+
+def _ask_email():
     if FANCY_USER_INPUT:
         account = inquirer.text('Email address', validate=lambda _, x: '@' in x)
     else:
@@ -29,16 +31,15 @@ def ask_login_info() -> str:
             account = Prompt.ask('[[yellow]?[/yellow]] Email address')
             if '@' in account:
                 break
+            else:
+                console.print(f"'{account}' is not a valid")
 
-    response = piperider_cloud.magic_login(account)
-    if response is None or response.get('success') is False:
-        console.print('[[red]Error[/red]] Login failed. Please try again.')
-        return None
+    return account
 
-    if response.get('link'):
-        webbrowser.open(response.get('link'))
 
-    console.print('Please paste the api token from magic link. The link has been sent to your email address.')
+def _ask_api_token():
+    console.print('We have just sent an email with a magic link.\n'
+                  'Please paste the api token from the magic link.')
     while True:
         api_token = Prompt.ask('[[yellow]?[/yellow]] API token')
         if len(api_token) > 0:
@@ -46,6 +47,76 @@ def ask_login_info() -> str:
     return api_token
 
 
+def _process_magic_signup(account: str, username: str = None):
+    response = piperider_cloud.magic_signup(account, username)
+    if response is None:
+        console.print('[[red]Error[/red]] Signup failed. Please try again.')
+        return False
+    elif response.get('success') is False:
+        # TODO: use error code
+        if 'User email or username already exists.' in response.get('message', ''):
+            console.print(f"'{account}' already exists.")
+            if not _process_magic_login(account):
+                return False
+        else:
+            console.print(f"[[red]Error[/red]] {response.get('message')}")
+            return False
+    return True
+
+
+def _process_magic_login(account: str):
+    response = piperider_cloud.magic_login(account)
+    if response is None:
+        console.print('[[red]Error[/red]] Login failed. Please try again.')
+        return False
+    elif response.get('success') is False:
+        # TODO: use error code
+        if f"'{account}' does not exist." in response.get('message', ''):
+            console.print('This email account was not found.')
+            question = f"Signup with '{account}' (y/n)"
+            allowed_ans = ['y', 'n', 'Y', 'N']
+            if FANCY_USER_INPUT:
+                ans = inquirer.text(question, validate=lambda _, x: x in allowed_ans)
+            else:
+                while True:
+                    ans = Prompt.ask(f'[[yellow]?[/yellow]] {question}')
+                    if ans in allowed_ans:
+                        break
+                    else:
+                        console.print(f"'{ans}' is not a valid")
+            if ans.upper() == 'N':
+                return False
+            else:
+                if _process_magic_signup(account):
+                    return True
+                return False
+        else:
+            console.print(f"[[red]Error[/red]] {response.get('message')}")
+            return False
+
+    if response.get('link'):
+        webbrowser.open(response.get('link'))
+    return True
+
+
+def ask_signup_info() -> str:
+    console.print('Please provide your email account to signup')
+    account = _ask_email()
+
+    if _process_magic_signup(account):
+        return _ask_api_token()
+    return ''
+
+
+def ask_login_info() -> str:
+    console.print('Please provide your email account to login')
+    account = _ask_email()
+
+    if _process_magic_login(account):
+        return _ask_api_token()
+    return ''
+
+
 def verify_api_token(api_token) -> bool:
     if piperider_cloud.validate(api_token) is False:
         console.print('[[red]Error[/red]] Invalid API Token. Please try again.')
@@ -87,26 +158,19 @@ def show_user_info():
     pass
 
 
-def check_default_config(options: dict):
-    if options is None:
-        return
+def setup_cloud_default_config(options: dict = None):
+    if piperider_cloud.config.get('auto_upload') is None:
+        piperider_cloud.update_config({'auto_upload': False})
 
-    cloud_config = piperider_cloud.config
-
-    auto_upload_flag = options.get('auto_upload')
-
-    if cloud_config.get('auto_upload') is not None and auto_upload_flag is None:
-        # Skip if auto_upload is already set
-        return
-    if auto_upload_flag is None and cloud_config.get('auto_upload') is None:
-        console.print('Please select default behavior for auto upload')
-        if FANCY_USER_INPUT:
-            auto_upload_flag = inquirer.confirm('Auto upload reports to cloud', default=True)
+    if options and options.get('default_project'):
+        project_name = options.get('default_project')
+        if piperider_cloud.get_project_by_name(project_name):
+            piperider_cloud.update_config({'default_project': project_name})
+            console.print(f'[[bold green]Config[/bold green]] Default project is set to \'{project_name}\'')
         else:
-            auto_upload_flag = Prompt.ask('[[yellow]?[/yellow]] Auto upload reports to cloud', choices=['y', 'n'],
-                                          default='y') == 'y'
-    console.print(f'[[bold green]Config[/bold green]] Default auto upload behavior is set to {auto_upload_flag}')
-    piperider_cloud.update_config({'auto_upload': auto_upload_flag})
+            console.print(f"[[yellow]Skip[/yellow]] Project '{project_name}' does not exist.")
+    if piperider_cloud.config.get('default_project') is None:
+        CloudConnector.select_project()
 
 
 def select_reports(report_dir=None, datasource=None) -> List[RunOutput]:
@@ -127,17 +191,17 @@ class CloudReportOutput(RunOutput):
         self.fail_count = report['failed']
 
 
-def select_cloud_report_ids(project_id: int = None, datasource=None, project_name=None, target=None, base=None) -> (
-    int, int):
+def select_cloud_report_ids(datasource=None, project: PipeRiderProject = None, target=None,
+                            base=None) -> (int, int):
+    if project is None:
+        project = piperider_cloud.get_default_project()
+
     if target:
-        target_id = get_run_report_id(target)
+        target_id = get_run_report_id(project, target)
     if base:
-        base_id = get_run_report_id(base)
+        base_id = get_run_report_id(project, base)
 
-    if project_id is None:
-        project_id = piperider_cloud.get_default_project()
-
-    reports = [CloudReportOutput(r) for r in piperider_cloud.list_reports(project_id, datasource=datasource)]
+    reports = [CloudReportOutput(r) for r in piperider_cloud.list_reports(project, datasource=datasource)]
     if len(reports) == 0:
         return None, None
 
@@ -160,22 +224,33 @@ def select_cloud_report_ids(project_id: int = None, datasource=None, project_nam
     return base_id, target_id
 
 
-def upload_to_cloud(report: RunOutput, debug=False, project_id=None) -> dict:
-    response = piperider_cloud.upload_report(report.path, project_id=project_id)
+def upload_to_cloud(run: RunOutput, debug=False, project: PipeRiderProject = None, show_progress=True) -> dict:
+    response = piperider_cloud.upload_run(run.path, project=project, show_progress=show_progress)
+
     # TODO refine the output when API is ready
 
+    def _patch_cloud_upload_response(run_path, project: PipeRiderProject, run_id):
+        with open(run_path, 'r') as f:
+            report = json.load(f)
+        report['cloud'] = {
+            'run_id': run_id,
+            'project_name': f'{project.workspace_name}/{project.name}'
+        }
+        with open(run_path, 'w') as f:
+            f.write(json.dumps(report, separators=(',', ':')))
+
     if response.get('success') is True:
-        project_id = response.get('project_id')
-        report_id = response.get('id')
-        if project_id and report_id:
-            report_url = f'{piperider_cloud.service.cloud_host}/projects/{project_id}/reports/{report_id}'
+        run_id = response.get('id')
+        if run_id:
+            report_url = f'{piperider_cloud.service.cloud_host}/{project.workspace_name}/{project.name}/runs/{run_id}'
+            _patch_cloud_upload_response(run.path, project, run_id)
         else:
             report_url = 'N/A'
         return {
             'success': True,
             'message': response.get("message"),
             'report_url': report_url,
-            'file_path': report.path
+            'file_path': run.path
         }
 
     if debug:
@@ -184,11 +259,11 @@ def upload_to_cloud(report: RunOutput, debug=False, project_id=None) -> dict:
         'success': False,
         'message': response.get("message"),
         'report_url': 'N/A',
-        'file_path': report.path
+        'file_path': run.path
     }
 
 
-def get_run_report_id(report_key: str) -> Optional[int]:
+def get_run_report_id(project: PipeRiderProject, report_key: str) -> Optional[int]:
     if report_key.isdecimal():
         if int(report_key) < 1:
             return None
@@ -196,8 +271,7 @@ def get_run_report_id(report_key: str) -> Optional[int]:
 
     if report_key.startswith('datasource:'):
         datasource = report_key.split(':')[-1]
-        project_id = piperider_cloud.get_default_project()
-        reports = piperider_cloud.list_reports(project_id, datasource=datasource)
+        reports = piperider_cloud.list_reports(project, datasource=datasource)
 
         if reports:
             return reports[0].get('id')
@@ -205,15 +279,14 @@ def get_run_report_id(report_key: str) -> Optional[int]:
     return None
 
 
-def create_compare_reports(base_id: int, target_id: int, tables_from, project_id=None) -> dict:
-    if project_id is None:
-        project_id = piperider_cloud.get_default_project()
-    response = piperider_cloud.compare_reports(project_id, base_id, target_id, tables_from)
-    if response is not None:
-        url = f'{piperider_cloud.service.cloud_host}/projects/{project_id}/reports/{base_id}/comparison/{target_id}'
-        console.print(f'Comparison report URL: {url}')
-    else:
-        console.print('Failed to create the comparison report')
+def create_compare_reports(base_id: int, target_id: int, tables_from, project: PipeRiderProject = None) -> dict:
+    if project is None:
+        project = piperider_cloud.get_default_project()
+    response = piperider_cloud.compare_reports(base_id, target_id, tables_from, project=project)
+    if response:
+        url = f'{piperider_cloud.service.cloud_host}/{project.workspace_name}/{project.name}/runs/{base_id}/comparison/{target_id}'
+        response['url'] = url
+
     return response
 
 
@@ -231,13 +304,34 @@ class CloudConnector:
         console.print(f'[[bold green]Config[/bold green]] Default auto upload behavior is set to {flag}')
         piperider_cloud.update_config({'auto_upload': flag})
 
+    @staticmethod
+    def signup(debug=False):
+
+        if piperider_cloud.available:
+            console.rule('Already Logged In')
+            show_user_info()
+            setup_cloud_default_config()
+            return 0
+
+        api_token = ask_signup_info()
+
+        if api_token:
+            if verify_api_token(api_token):
+                console.rule('Login Successful')
+                show_user_info()
+                setup_cloud_default_config()
+                return 0
+
+        console.rule('Login Failed', style='red')
+        return 1
+
     @staticmethod
     def login(api_token=None, options: dict = None, debug=False):
 
         if piperider_cloud.available:
             console.rule('Already Logged In')
             show_user_info()
-            check_default_config(options)
+            setup_cloud_default_config(options)
             return 0
 
         if api_token is None:
@@ -247,7 +341,7 @@ class CloudConnector:
             if verify_api_token(api_token):
                 console.rule('Login Successful')
                 show_user_info()
-                check_default_config(options)
+                setup_cloud_default_config(options)
                 return 0
 
         console.rule('Login Failed', style='red')
@@ -266,73 +360,102 @@ class CloudConnector:
 
     @staticmethod
     def upload_latest_report(report_dir=None, debug=False, open_report=False, force_upload=False,
-                             auto_upload=False) -> int:
+                             auto_upload=False, project_name: str = None) -> int:
         filesystem = FileSystem(report_dir=report_dir)
         latest_report_path = os.path.join(filesystem.get_output_dir(), 'latest', 'run.json')
         return CloudConnector.upload_report(latest_report_path, debug=debug, open_report=open_report,
-                                            force_upload=force_upload, auto_upload=auto_upload)
+                                            force_upload=force_upload, auto_upload=auto_upload,
+                                            project_name=project_name)
 
     @staticmethod
     def upload_report(report_path=None, report_dir=None, datasource=None, debug=False, open_report=False,
-                      force_upload=False, auto_upload=False, project_name=None) -> int:
+                      force_upload=False, auto_upload=False, project_name=None, show_progress=True) -> int:
         if piperider_cloud.available is False:
             console.rule('Please login PipeRider Cloud first', style='red')
             return 1
 
-        project_id = piperider_cloud.get_default_project()
-        if project_name is not None:
+        if project_name:
             project = piperider_cloud.get_project_by_name(project_name)
             if project is None:
                 console.print(f'[[bold red]Error[/bold red]] Project \'{project_name}\' does not exist')
                 return 1
-            project_id = project.get('id')
+        else:
+            project = piperider_cloud.get_default_project()
 
         rc = 0
         results = []
         if report_path is None:
             reports = select_reports(report_dir=report_dir, datasource=datasource)
-            console.rule('Uploading Reports')
+            if show_progress:
+                console.rule('Uploading Reports')
             for r in reports:
-                response = upload_to_cloud(r, debug, project_id=project_id)
+                response = upload_to_cloud(r, debug, project=project, show_progress=show_progress)
                 if response.get('success') is False:
                     rc = 1
                 response['name'] = r.name
                 response['created_at'] = r.created_at
                 results.append(response)
         else:
-            console.rule('Uploading Report')
+            if show_progress:
+                console.rule('Uploading Report')
             report = RunOutput(report_path)
-            response = upload_to_cloud(report, debug, project_id=project_id)
+            response = upload_to_cloud(report, debug, project=project, show_progress=show_progress)
             if response.get('success') is False:
                 rc = 1
             response['name'] = report.name
             response['created_at'] = report.created_at
             results.append(response)
 
-        console.rule('Upload Completed')
-        ascii_table = Table(show_header=True, show_edge=True, header_style="bold magenta",
-                            box=box.SIMPLE)
-        ascii_table.add_column('Status', justify='left', style='cyan')
-        ascii_table.add_column('Name', justify='left')
-        ascii_table.add_column('Created At', justify='left')
-        ascii_table.add_column('Report URL', justify='left', no_wrap=True)
-        ascii_table.add_column('Message', justify='left')
+        if show_progress:
+            console.rule('Upload Completed')
+            ascii_table = Table(show_header=True, show_edge=True, header_style="bold magenta",
+                                box=box.SIMPLE)
+            ascii_table.add_column('Status', justify='left', style='cyan')
+            ascii_table.add_column('Name', justify='left')
+            ascii_table.add_column('Created At', justify='left')
+            ascii_table.add_column('Report URL', justify='left', no_wrap=True)
+            ascii_table.add_column('Message', justify='left')
 
-        for response in results:
-            status = '[bold green]Success[/bold green]' if response.get(
-                'success') else '[bold yellow]Skipped[/bold yellow]'
-            url = f"[deep_sky_blue1]{response.get('report_url', 'N/A')}[/deep_sky_blue1]"
-            message = response.get('message')
-            created_at = datetime_to_str(str_to_datetime(response.get('created_at')), to_tzlocal=True)
-            ascii_table.add_row(status, response.get('name'), created_at, url, message)
+            for response in results:
+                status = '[bold green]Success[/bold green]' if response.get(
+                    'success') else '[bold yellow]Skipped[/bold yellow]'
+                url = f"[deep_sky_blue1]{response.get('report_url', 'N/A')}[/deep_sky_blue1]"
+                message = response.get('message')
+                created_at = datetime_to_str(str_to_datetime(response.get('created_at')), to_tzlocal=True)
+                ascii_table.add_row(status, response.get('name'), created_at, url, message)
+            console.print(ascii_table)
 
         if open_report:
             url = response.get('report_url')
             open_report_in_browser(url, True)
 
-        console.print(ascii_table)
         return rc
 
+    @staticmethod
+    def generate_compare_report(base_id: str, target_id: str, tables_from='all',
+                                project_name: str = None, debug: bool = False):
+        # TODO: Change to use new front-end URL pattern
+        def _generate_legacy_compare_report_url(base_id, target_id, project=None):
+            if project is None:
+                project = piperider_cloud.get_default_project()
+            response = create_compare_reports(base_id, target_id, tables_from, project=project)
+            return response
+
+        try:
+            # TODO: change to use workspace and project name instead of project id
+            if project_name:
+                project = piperider_cloud.get_project_by_name(project_name)
+                if project is None:
+                    console.print(f'[[bold red]Error[/bold red]] Project \'{project_name}\' does not exist')
+                    return 1
+            else:
+                project = piperider_cloud.get_default_project()
+            return _generate_legacy_compare_report_url(base_id, target_id, project=project)
+        except Exception as e:
+            if debug:
+                console.print(f'[[bold red]Error[/bold red]] {e}')
+            return None
+
     @staticmethod
     def compare_reports(base=None, target=None, tables_from='all', summary_file=None, debug=False,
                         project_name=None) -> int:
@@ -340,20 +463,27 @@ class CloudConnector:
             console.rule('Please login PipeRider Cloud first', style='red')
             return 1
 
-        project_id = None
-        if project_name is not None:
+        if project_name:
             project = piperider_cloud.get_project_by_name(project_name)
             if project is None:
                 console.print(f'[[bold red]Error[/bold red]] Project \'{project_name}\' does not exist')
                 return 1
-            project_id = project.get('id')
+        else:
+            project = piperider_cloud.get_default_project()
 
-        base_id, target_id = select_cloud_report_ids(base=base, target=target, project_id=project_id)
+        base_id, target_id = select_cloud_report_ids(base=base, target=target, project=project)
         if base_id is None or target_id is None:
-            raise Exception('No report found in the PipeRider Cloud. Please upload reports to PipeRider Cloud first.')
+            raise Exception(
+                f'No reports found in the project: {project.workspace_name}/{project.name}. '
+                f'Please upload reports to PipeRider Cloud first.')
 
         console.print(f"Creating comparison report id={base_id} ... id={target_id}")
-        response = create_compare_reports(base_id, target_id, tables_from, project_id=project_id)
+        response = create_compare_reports(base_id, target_id, tables_from, project=project)
+        if response is None:
+            console.print('Failed to create the comparison report')
+        else:
+            url = response.get('url')
+            console.print(f'Comparison report URL: {url}')
 
         if debug:
             console.print(response)
@@ -366,6 +496,31 @@ class CloudConnector:
             with open(summary_file, 'w') as f:
                 f.write(response.get('summary'))
 
+    @staticmethod
+    def share_compare_report(base_id=None, target_id=None, debug=False):
+        if piperider_cloud.available is False:
+            console.rule('Please login PipeRider Cloud first', style='red')
+            return 1
+
+        workspace_name, project_name = piperider_cloud.get_default_workspace_and_project()
+        if workspace_name is None or project_name is None:
+            console.rule('Please select a workspace and a project first', style='red')
+            return 1
+
+        if debug:
+            console.print(f"Sharing comparison report id={base_id} ... id={target_id}")
+        response = piperider_cloud.share_compare_report(
+            workspace_name=workspace_name,
+            project_name=project_name,
+            base_id=base_id,
+            target_id=target_id,
+        )
+        sharing_url = None
+
+        if response:
+            sharing_url = f'{piperider_cloud.service.cloud_host}/runs/sharing/comparison/{response.get("sharing_id")}'
+        return sharing_url
+
     @staticmethod
     def list_projects(debug=False) -> int:
         if piperider_cloud.available is False:
@@ -383,8 +538,7 @@ class CloudConnector:
             box=box.SIMPLE_HEAVY,
         )
         layout_table.add_column('Name')
-        layout_table.add_column('Type')
-        layout_table.add_column('Organization')
+        layout_table.add_column('Workspace')
         # layout_table.add_column('Project URL', justify='right', no_wrap=True)
 
         for project in projects:
@@ -392,11 +546,13 @@ class CloudConnector:
             # project_id = project.get('id')
             # project_url = f'[deep_sky_blue1]{piperider_cloud.service.cloud_host}/projects/{project_id}[/deep_sky_blue1]'
             project_name = project.get('name') if project.get(
-                'organization_name') is None else f"{project.get('organization_name')}/{project.get('name')}"
+                'workspace_name') is None else f"{project.get('workspace_name')}/{project.get('name')}"
+            workspace_name = project.get('workspace_name')
+            display_name = project.get('workspace_display_name') if project.get(
+                'workspace_display_name') else workspace_name
             layout_table.add_row(
                 project_name,
-                project.get('parent_type'),
-                project.get('organization_display_name', '-'),
+                display_name
                 # project_url,
             )
 
@@ -404,7 +560,7 @@ class CloudConnector:
         pass
 
     @staticmethod
-    def select_project(project_name: str = None, datasource: str = None, debug: bool = False):
+    def select_project(project_name: str = None, datasource: str = None, debug: bool = False) -> int:
 
         def _project_selector():
             arrow_alias_msg = ''
@@ -415,9 +571,14 @@ class CloudConnector:
                 arrow_alias_msg = " 'w' to Up, 's' to Down,"
 
             projects = [
-                (f"{p.get('organization_name')}/{p.get('name')}" if p.get('organization_name') else p.get('name'), p)
+                (f"{p.get('workspace_name')}/{p.get('name')}" if p.get('workspace_name') else p.get('name'),
+                 PipeRiderProject(p))
                 for p in piperider_cloud.list_projects()]
 
+            if len(projects) == 1:
+                _, p = projects[0]
+                return p
+
             question = [
                 inquirer.List('selected_project',
                               message=f"Please select a project as default project ({arrow_alias_msg} ENTER to confirm )",
@@ -438,10 +599,9 @@ class CloudConnector:
                 console.print(f'[[bold red]Warning[/bold red]] Project \'{project_name}\' does not exist')
                 return 1
 
-        name = project.get('name') if project.get(
-            'organization_name') is None else f"{project.get('organization_name')}/{project.get('name')}"
+        name = project.name if project.workspace_name is None else f"{project.workspace_name}/{project.name}"
         piperider_cloud.set_default_project(name)
 
         # TODO: Add project name into the datasource config if datasource is not None
         console.print(f'[[bold green]Config[/bold green]] Default project is set to \'{name}\'')
-        pass
+        return 0
